package wekagyak;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Random;

import weka.associations.Apriori;
import weka.classifiers.Evaluation;
import weka.classifiers.trees.J48;
import weka.clusterers.ClusterEvaluation;
import weka.clusterers.SimpleKMeans;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

public class Main {

	public static void main(String[] args) {
		Instances data = readFile("tic-tac-toe.arff");
		data.setClassIndex(data.numAttributes()-1);
		Evaluation eval = doClassification(data); 
		if(eval != null) {
			System.out.println(eval.toSummaryString());
		}
		//confusion matrix (we don't do anything with it now):
		double[][] matrix = eval.confusionMatrix();
		
		
		Instances clusterdata = filterTargetAttribute(data);
		ClusterEvaluation clusterEval = doClustering(clusterdata, data);
		if(clusterEval != null) {
			System.out.println(clusterEval.clusterResultsToString());
		}
		
		
		Apriori apriori = doAssociationMining(data);
		if(apriori != null) {
			System.out.println(apriori.toString());
		}
	}
	
	public static Instances readFile(String path) {
		Instances data = null;
		FileReader fr;		
		try {
			fr = new FileReader(path);
			BufferedReader br = new BufferedReader(fr);
			data = new Instances(br);
			br.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		return data;
	}
	
	public static Evaluation doClassification(Instances data) {
		Evaluation eval = null;
		J48 tree = new J48();
		tree.setBinarySplits(false);
		tree.setMinNumObj(13);
		tree.setReducedErrorPruning(true);
		try {
			tree.buildClassifier(data);
			eval = new Evaluation(data);
			eval.crossValidateModel(tree, data, 10, new Random());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return eval;
	}
	
	public static ClusterEvaluation doClustering(Instances clusterdata, Instances original) {
		ClusterEvaluation result = null;
		SimpleKMeans skm = new SimpleKMeans();
		try {
			skm.setNumClusters(2);
			skm.buildClusterer(clusterdata);
			
			result = new ClusterEvaluation();
			result.setClusterer(skm);
			result.evaluateClusterer(original);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return result;		
	}
	
	public static Apriori doAssociationMining(Instances data) {
		
		Apriori apriori = null;
		try{
			apriori = new Apriori();
			String[] options = {"-M", "0.1", "-C", "0.5", "-N", "200"};
			apriori.setOptions(options);
			apriori.buildAssociations(data);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		return apriori;
	}
	
	public static Instances filterTargetAttribute(Instances data) {
		Instances result = null;
		Remove filter = new Remove();
		filter.setAttributeIndicesArray(new int[] {data.classIndex()});
		try {
			filter.setInputFormat(data);
			result = Filter.useFilter(data,filter);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

}
